/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.esb;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
/*
 * 
 */

public class ServiceBusController implements Runnable {
// use the local loopback address so that only localhost can send the messages
    public static final String LOCAL_HOST_IP = "127.0.0.1";
    public static final String SHUTDOWN_COMMAND = "shutdownserver";

    // ServiceBus instance
    private ServiceBus esb;
    private int port;

    private Log log = LogFactory.getLog(ServiceBusController.class);

    public ServiceBusController(ServiceBus esb) {
        this.esb = esb;
    }

    public void run() {
        waitForShutDownCommand();
    }

    private void waitForShutDownCommand() {
        try {
            ServerSocket serverSocket = new ServerSocket(port, 1,
                    InetAddress.getByName(
                            LOCAL_HOST_IP));
            // wait here until some one talk to server
            log.debug("Listening to port " + this.port);
            Socket socket = serverSocket.accept();
            InputStream inputStream = socket.getInputStream();
            StringBuffer stringBuffer = new StringBuffer();
            int ch;
            while ((ch = inputStream.read()) != -1) {
                stringBuffer.append((char) ch);
            }

            String command = stringBuffer.toString();
            log.debug("Received Command ==> " + command);
            if (command.equals(SHUTDOWN_COMMAND)) {
                // close the socket and exit from the jvm
                inputStream.close();
                socket.close();
                serverSocket.close();
                //this calls for shutdownhook and it stops the service
                System.exit(1);

            } else {
                log.error("Invalid command");
            }

        } catch (IOException e) {
            e.printStackTrace();
            // if we can not start the server then we can not run the server
            log.error("Can not listen to the port. Port Already used");
            System.exit(1);
        }

    }

    /**
     * send the shutdown command to server listen port
     */

    public void shutDownServer() {
        try {
            Socket socket = new Socket(LOCAL_HOST_IP, port);
            OutputStream outStream = socket.getOutputStream();
            for (int i = 0; i < SHUTDOWN_COMMAND.length(); i++) {
                outStream.write(SHUTDOWN_COMMAND.charAt(i));
            }
            outStream.flush();
            outStream.close();
            socket.close();
        } catch (IOException e) {
            e.printStackTrace();
            log.error("No runnig server to close..");
            System.exit(1);
        }
    }

    public int getPort() {
        return port;
    }

    public void setPort(int port) {
        this.port = port;
    }
}
