/*
* Copyright 2004,2005 The Apache Software Foundation.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.wso2.esb.persistence;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.esb.persistence.dao.StatisticsDAO;
import org.wso2.esb.persistence.dao.RegistryEntryDAO;
import org.wso2.esb.persistence.dataobject.StatisticsDO;
import org.wso2.esb.persistence.dataobject.RegistryEntryDO;
import org.wso2.esb.util.HibernateConfig;
import org.wso2.esb.util.HibernateConfigFactory;

import java.util.List;

/**
 *
 */
public class PersistenceManager {

    private static Log log = LogFactory.getLog(PersistenceManager.class);

    /** The HibernateConfiguration */
    private HibernateConfig hbConfig;

    public PersistenceManager() {
        this.hbConfig = HibernateConfigFactory.getDefaultConfig(
            org.wso2.esb.Constants.WSO2ESB_HB_CONFIG_KEY, "wso2esb.hibernate.cfg.xml");
    }

    public PersistenceManager(HibernateConfig hbConfig) {
        this.hbConfig = hbConfig;
    }

    /**
     * add a Statistics Record to the database
     *
     * @param statisticsDO
     */
    public void addStatisticsRecord(StatisticsDO statisticsDO) {
        new StatisticsDAO(hbConfig).create(statisticsDO);
    }

    /**
     * delete a Statistics Record
     *
     * @param statisticsDO
     */
    public void deleteStatisticsRecord(StatisticsDO statisticsDO) {
        new StatisticsDAO(hbConfig).delete(statisticsDO);
    }

    /**
     * update a Satistics Record
     *
     * @param statisticsDO
     */
    public void updateStatisticsRecord(StatisticsDO statisticsDO) {
        new StatisticsDAO(hbConfig).update(statisticsDO);
    }

    /**
     * Retrieves Statistics Records
     *
     * @param hqlQuery
     * @return The Statistics Records
     */
    public StatisticsDO[] selectStatisticsRecords(String hqlQuery) {
        return new StatisticsDAO(hbConfig).selectStatisticsRecords(hqlQuery);
    }

    /**
     * Retrieves Statistics Records using custom SQL query passed in
     *
     * @param sqlQuery
     * @param aClass
     * @return The Statistics Records
     */
    public StatisticsDO[] selectStatisticsRecordsBySQL(String sqlQuery, Class aClass) {
        return new StatisticsDAO(hbConfig).selectStatisticsRecordsBySQL(sqlQuery, aClass);
    }

    /**
     * To create a new object based on Statistics Records
     *
     * @param hqlQuery
     * @return new result object
     */
    public Object createAnewObjectFromStatistics(String hqlQuery) {
        return new StatisticsDAO(hbConfig).createAnewObject(hqlQuery);
    }

    public List selectCustomObjectList(String hqlQuery) {
            return new StatisticsDAO(hbConfig).selectCustomObjectList(hqlQuery);
    }

    public int clearAll(String hglQuery){
        return  new StatisticsDAO(hbConfig).execute(hglQuery);
    }

    /* Data access methods for ESB registry */
    public void addRegistryEntry(RegistryEntryDO registryEntryDO) {
        new RegistryEntryDAO(hbConfig).addRegistryEntry(registryEntryDO);
    }

    public void updateRegistryEntry(RegistryEntryDO registryEntryDO) {
        new RegistryEntryDAO(hbConfig).updateRegistryEntry(registryEntryDO);
    }

    public void saveOrUpdateRegistryEntry(RegistryEntryDO registryEntryDO) {
        new RegistryEntryDAO(hbConfig).saveOrUpdateRegistryEntry(registryEntryDO);
    }

    public RegistryEntryDO getRegistryEntry(String key) {
        return new RegistryEntryDAO(hbConfig).getRegistryEntry(key);
    }

    public void deleteRegistryEntry(String key) {
        new RegistryEntryDAO(hbConfig).deleteRegistryEntry(key);
    }
}
