/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.esb.services;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.engine.AxisConfiguration;
import org.apache.synapse.config.SynapseConfiguration;
import org.apache.commons.logging.Log;

/*
 *  Abstract Class that handles Admin stuff
 */

public abstract class AbstractESBAdmin {
    public static String LOGGED = "logged";
    
    public AxisConfiguration getAxisConfig() {
        return MessageContext.getCurrentMessageContext().getConfigurationContext()
                .getAxisConfiguration();
    }

    public ConfigurationContext getConfigContext() {
        return MessageContext.getCurrentMessageContext().getConfigurationContext();
    }

    public SynapseConfiguration getSynapseConfiguration() throws AxisFault {
        Parameter synapseConfigParam = getAxisConfig().getParameter(
                org.apache.synapse.Constants.SYNAPSE_CONFIG);
        if (synapseConfigParam == null) {
            throw new AxisFault("Unable to find the Synapse configuration from" +
                " the loaded Axis2 configuration");
        }
        return (SynapseConfiguration) synapseConfigParam.getValue();
    }

    protected void handleFault(Log log, String message, Exception e) throws AxisFault {

        if (e == null) {
            AxisFault af = new AxisFault(message);
            log.error(message, af);
            throw af;

        } else {
            message = message + " :: " + e.getMessage(); 
            log.error(message, e);
            throw new AxisFault(message, e);
        }
    }
}
