/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.esb.services;

import org.apache.axis2.AxisFault;
import org.apache.log4j.Appender;
import org.apache.log4j.Layout;
import org.apache.log4j.Logger;
import org.apache.log4j.spi.LoggingEvent;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.esb.ServiceBusException;
import org.wso2.esb.services.tos.ServerStatus;
import org.wso2.esb.services.tos.ServerData;
import org.wso2.utils.MemoryAppender;

import java.util.List;
import java.util.ArrayList;

/*
 * Returns ServerStatus for the ESB home page
 * Detailed ServerData for the Manage->System page
 * Tracing and Log entry views off the MemoryAppender
 */

public class ServerAdmin extends AbstractESBAdmin {

    private static Log log = LogFactory.getLog(ServerAdmin.class);

    public ServerStatus getStatus() throws AxisFault {
        return new ServerStatus(getAxisConfig());
    }

    public ServerData getServerData() throws AxisFault {
        return new ServerData(
                org.wso2.esb.Constants.ESB_INSTANCE,
                getAxisConfig().getRepository().getPath());
    }

    public String[] getTraceLogs() throws ServiceBusException {
        int amount;
        int DEFAULT_NO_OF_LOGS = 100;
        Logger logger = Logger.getLogger(org.apache.synapse.Constants.TRACE_LOGGER);
        Appender appender = logger.getAppender("TRACE_MEMORYAPPENDER");
        if (appender instanceof MemoryAppender) {
            MemoryAppender memoryAppender = (MemoryAppender) appender;
            if ((memoryAppender.getCircularQueue() != null)) {
                amount = memoryAppender.getBufferSize();
            } else {
                return new String[]{
                        "--- No trace entries found. " +
                                "You need to enable tracing on the configuration elements ---"};
            }
            if ((memoryAppender.getCircularQueue().getObjects(amount) == null) ||
                    (memoryAppender.getCircularQueue().getObjects(amount).length == 0)) {
                return new String[]{
                        "--- No trace entries found. " +
                                "You need to enable tracing on the configuration elements ---"};
            }
            Object[] objects;
            if (amount < 1) {
                objects = memoryAppender.getCircularQueue().getObjects(DEFAULT_NO_OF_LOGS);
            } else {
                objects = memoryAppender.getCircularQueue().getObjects(amount);
            }
            String[] resp = new String[objects.length];
            Layout layout = memoryAppender.getLayout();
            for (int i = 0; i < objects.length; i++) {
                LoggingEvent logEvt = (LoggingEvent) objects[i];
                if (logEvt != null) {
                    resp[i] = layout.format(logEvt);
                }
            }
            return resp;
        } else {
            return new String[]{"The trace log must be configured to use the " +
                    "org.wso2.utils.MemoryAppender to view entries through the admin console"};
        }
    }

    public boolean clearTraceLogs() {
        Logger logger = Logger.getLogger(org.apache.synapse.Constants.TRACE_LOGGER);
        Appender appender = logger.getAppender("TRACE_MEMORYAPPENDER");
        if (appender instanceof MemoryAppender) {
            try {
                MemoryAppender memoryAppender = (MemoryAppender) appender;
                if (memoryAppender.getCircularQueue() != null) {
                    memoryAppender.getCircularQueue().clear();
                }
                return true;
            } catch (Exception e) {
                return false;
            }
        } else {
            return false;
        }
    }

    public String[] getLogs() throws AxisFault, ServiceBusException {
        int DEFAULT_NO_OF_LOGS = 100;
        int definedamount;
        Appender appender
                = Logger.getRootLogger().getAppender("LOG_MEMORYAPPENDER");
        if (appender instanceof MemoryAppender) {
            MemoryAppender memoryAppender = (MemoryAppender) appender;
            if ((memoryAppender.getCircularQueue() != null)) {
                definedamount = memoryAppender.getBufferSize();
            } else {
                return new String[]{
                        "--- No log entries found. " +
                                "You may try increasing the log level ---"
                };
            }
            if ((memoryAppender.getCircularQueue().getObjects(definedamount) == null) ||
                    (memoryAppender.getCircularQueue().getObjects(definedamount).length == 0)) {
                return new String[]{
                        "--- No log entries found. " +
                                "You may try increasing the log level ---"
                };
            }
            Object[] objects;
            if (definedamount < 1) {
                objects = memoryAppender.getCircularQueue().getObjects(DEFAULT_NO_OF_LOGS);
            } else {
                objects = memoryAppender.getCircularQueue().getObjects(definedamount);
            }
            String[]  resp = new String[objects.length];
            Layout layout = memoryAppender.getLayout();
            for (int i = 0; i < objects.length; i++) {
                LoggingEvent logEvt = (LoggingEvent) objects[i];
                if (logEvt != null) {
                    resp[i] = layout.format(logEvt);
                }
            }
            return resp;
        } else {
            return new String[]{"The log must be configured to use the " +
                    "org.wso2.utils.MemoryAppender to view entries on the admin console"};
        }
    }

    public String[] getLogsForSingleLogLevel(String type) throws AxisFault, ServiceBusException {
        if (type == null) {
            handleFault(log, "Type can not be null", null);
        }
        if ("ALL".equals(type)) {
            return getLogs();
        } else {
            return getLogsForType(type);
        }
    }

    public String[] searchLog(String type, String keyword) throws AxisFault, ServiceBusException {
        if (keyword == null) {
            handleFault(log, "Key word can not be null", null);
        }
        if (type == null) {
            handleFault(log, "Type can not be null", null);
        }
        if ("ALL".equals(type)) {
            if ("".equals(keyword)) {
                return getLogs();
            } else {
                return getLogsForKey(keyword);
            }
        } else {
            String[] filerByType = getLogsForType(type);
            List resultList = new ArrayList();
            for (int i = 0; i < filerByType.length ; i++) {
                if (filerByType[i] != null && filerByType[i].indexOf(keyword) > -1) {
                    resultList.add(filerByType[i]);
                }
            }
            if (resultList.isEmpty()) {
                return new String[]{
                        "--- No log entries found for " +
                                "the keyword " + keyword + " and the log level " + type + "---"
                };
            }
            return (String[]) resultList.toArray(new String[resultList.size()]);
        }
    }

    private String [] getLogsForKey(String keyword) {
        int DEFAULT_NO_OF_LOGS = 100;
        int definedamount;
        Appender appender
                = Logger.getRootLogger().getAppender("LOG_MEMORYAPPENDER");
        if (appender instanceof MemoryAppender) {
            MemoryAppender memoryAppender = (MemoryAppender) appender;
            if ((memoryAppender.getCircularQueue() != null)) {
                definedamount = memoryAppender.getBufferSize();
            } else {
                return new String[]{
                        "--- No log entries found for " +
                                "the  " + keyword + " ---"
                };
            }
            if ((memoryAppender.getCircularQueue().getObjects(definedamount) == null) ||
                    (memoryAppender.getCircularQueue().getObjects(definedamount).length == 0)) {
                return new String[]{
                        "--- No log entries found for " +
                                "the  " + keyword + " ---"
                };
            }
            Object[] objects;
            if (definedamount < 1) {
                objects = memoryAppender.getCircularQueue().getObjects(DEFAULT_NO_OF_LOGS);
            } else {
                objects = memoryAppender.getCircularQueue().getObjects(definedamount);
            }
            Layout layout = memoryAppender.getLayout();
            List resultList = new ArrayList();
            for (int i = 0; i < objects.length; i++) {
                LoggingEvent logEvt = (LoggingEvent) objects[i];
                if (logEvt != null) {
                    String result = layout.format(logEvt);
                    if (result != null && result.indexOf(keyword) > -1) {
                        resultList.add(result);
                    }
                }
            }
            if (resultList.isEmpty()) {
                return new String[]{
                        "--- No log entries found for " +
                                "the " + keyword + " ---"
                };
            }
            return (String[]) resultList.toArray(new String[resultList.size()]);
        } else {
            return new String[]{"The log must be configured to use the " +
                    "org.wso2.utils.MemoryAppender to view entries on the admin console"};
        }
    }

    private String [] getLogsForType(String type) {
        int DEFAULT_NO_OF_LOGS = 100;
        int definedamount;
        Appender appender
                = Logger.getRootLogger().getAppender("LOG_MEMORYAPPENDER");
        if (appender instanceof MemoryAppender) {
            MemoryAppender memoryAppender = (MemoryAppender) appender;
            if ((memoryAppender.getCircularQueue() != null)) {
                definedamount = memoryAppender.getBufferSize();
            } else {
                return new String[]{
                        "--- No log entries found for " +
                                "the  " + type + " ---"
                };
            }
            if ((memoryAppender.getCircularQueue().getObjects(definedamount) == null) ||
                    (memoryAppender.getCircularQueue().getObjects(definedamount).length == 0)) {
                return new String[]{
                        "--- No log entries found for " +
                                "the  " + type + " ---"
                };
            }
            Object[] objects;
            if (definedamount < 1) {
                objects = memoryAppender.getCircularQueue().getObjects(DEFAULT_NO_OF_LOGS);
            } else {
                objects = memoryAppender.getCircularQueue().getObjects(definedamount);
            }
            Layout layout = memoryAppender.getLayout();
            List resultList = new ArrayList();
            for (int i = 0; i < objects.length; i++) {
                LoggingEvent logEvt = (LoggingEvent) objects[i];
                if (logEvt != null) {
                    String result = layout.format(logEvt);
                    if (result != null && result.indexOf(type) > -1 && result.indexOf(type) < 4) {
                        resultList.add(result);
                    }
                }
            }
            if (resultList.isEmpty()) {
                return new String[]{
                        "--- No log entries found for " +
                                "the " + type + " ---"
                };
            }
            return (String[]) resultList.toArray(new String[resultList.size()]);
        } else {
            return new String[]{"The log must be configured to use the " +
                    "org.wso2.utils.MemoryAppender to view entries through the admin console"};
        }
    }

    public String[] searchTraceLog(String keyword) throws ServiceBusException {
        int DEFAULT_NO_OF_LOGS = 100;
        int definedamonut;
        if (keyword == null) {
            throw new ServiceBusException("Key word can not be null");
        }
        if ("ALL".equals(keyword) || "".equals(keyword)) {
            return getTraceLogs();
        }
        Logger logger = Logger.getLogger(org.apache.synapse.Constants.TRACE_LOGGER);
        Appender appender = logger.getAppender("TRACE_MEMORYAPPENDER");
        if (appender instanceof MemoryAppender) {
            MemoryAppender memoryAppender
                    = (MemoryAppender) appender;
            if ((memoryAppender.getCircularQueue() != null)) {
                definedamonut = memoryAppender.getBufferSize();
            } else {
                return new String[]{
                        "--- No trace entries found for " +
                                "the " + keyword + " ---"
                };
            }
            if ((memoryAppender.getCircularQueue().getObjects(definedamonut) == null) ||
                    (memoryAppender.getCircularQueue().getObjects(definedamonut).length == 0)) {
                return new String[]{
                        "--- No trace entries found for " +
                                "the " + keyword + " ---"
                };
            }
            Object[] objects;
            if (definedamonut < 1) {
                objects = memoryAppender.getCircularQueue().getObjects(DEFAULT_NO_OF_LOGS);
            } else {
                objects = memoryAppender.getCircularQueue().getObjects(definedamonut);
            }
            Layout layout = memoryAppender.getLayout();
            List resultList = new ArrayList();
            for (int i = 0; i < objects.length; i++) {
                LoggingEvent logEvt = (LoggingEvent) objects[i];
                if (logEvt != null) {
                    String result = layout.format(logEvt);
                    if (result != null && result.indexOf(keyword) > -1) {
                        resultList.add(result);
                    }
                }
            }
            if (resultList.isEmpty()) {
                return new String[]{
                        "--- No trace entries found for " +
                                "the  " + keyword + " ---"
                };
            }
            return (String[]) resultList.toArray(new String[resultList.size()]);
        } else {
            return new String[]{"The trace log must be configured to use the " +
                    "org.wso2.utils.MemoryAppender to view entries through the admin console"};
        }

    }

    public boolean clearLogs() {
        Appender appender = Logger.getRootLogger().getAppender("LOG_MEMORYAPPENDER");
        if (appender instanceof MemoryAppender) {
            try {
                MemoryAppender memoryAppender = (MemoryAppender) appender;
                if (memoryAppender.getCircularQueue() != null) {
                    memoryAppender.getCircularQueue().clear();
                }
                return true;
            } catch (Exception e) {
                return false;
            }
        } else {
            return false;
        }
    }


}
