/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.esb.services;

import org.apache.log4j.Logger;
import org.apache.synapse.Constants;
import org.wso2.esb.ServiceBusException;
import org.wso2.esb.persistence.PersistenceManager;
import org.wso2.esb.persistence.dataobject.StatisticsDO;
import org.wso2.esb.services.tos.GraphData;
import org.wso2.esb.services.tos.StatisticsData;
import org.wso2.esb.statistics.persistence.StatisticsDBUtils;
import org.wso2.esb.util.HibernateConfigFactory;

import java.util.Iterator;
import java.util.List;

/**
 *
 *
 */

public class ServiceAdmin extends AbstractESBAdmin {

    private static Logger log = Logger.getLogger(ServiceAdmin.class);

    private static PersistenceManager pm =
            new PersistenceManager(HibernateConfigFactory.getDefaultConfig(
                    org.wso2.esb.Constants.WSO2ESB_HB_CONFIG_KEY, "wso2esb.hibernate.cfg.xml"));


    private StatisticsData getData(int category, String serverId) {
        StatisticsData statisticsData = new StatisticsData();
        statisticsData.setCategory(category);
        statisticsData.setStatisticsRecords(
                StatisticsDBUtils.getStatisticsForServer(pm, serverId, category));
        return statisticsData;

    }

    private StatisticsData getMessageMediationData(String serverId) {

        StatisticsData statisticsData = new StatisticsData();
        statisticsData.setCategory(Constants.PROXYSERVICE_STATISTICS);
        statisticsData.setStatisticsRecords(StatisticsDBUtils.getStatistics(
                pm, serverId, Constants.PROXYSERVICE_STATISTICS, "SynapseService"));
        return statisticsData;
    }

    private StatisticsDO [] getRecords(int category, String name) {
        return StatisticsDBUtils.getStatistics(pm, category, name);
    }

    private StatisticsDO listToStatisticsDO(Object[] data) {
        StatisticsDO statisticsDO = new StatisticsDO();
        if (data != null && data.length == 5) {
            statisticsDO.setTotalCount(data[0] != null ? ((Long) data[0]).longValue() : 0);
            statisticsDO.setFaultCount(data[1] != null ? ((Long) data[1]).longValue() : 0);
            statisticsDO.setMaxTime(data[2] != null ? ((Float) data[2]).longValue() : 0);
            statisticsDO.setMinTime(data[3] != null ? ((Float) data[3]).longValue() : 0);
            statisticsDO.setAvgTime(data[4] != null ? ((Double) data[4]).doubleValue() : 0);
        }
        return statisticsDO;
    }

    private StatisticsDO mergeStatisticsDO(StatisticsDO statisticsDO1, StatisticsDO statisticsDO2) {
        if (statisticsDO1 == null && statisticsDO2 == null) {
            return null;
        } else if (statisticsDO2 == null) {
            return statisticsDO1;
        } else if (statisticsDO1 == null) {
            return statisticsDO2;
        } else {
            StatisticsDO statisticsDO = new StatisticsDO();
            statisticsDO.setTotalCount(
                    statisticsDO1.getTotalCount() + statisticsDO2.getTotalCount());
            statisticsDO.setFaultCount(
                    statisticsDO1.getFaultCount() + statisticsDO2.getFaultCount());
            statisticsDO.setMaxTime(statisticsDO1.getMaxTime() > statisticsDO2
                    .getMaxTime() ? statisticsDO1.getMaxTime() : statisticsDO2.getMaxTime());
            statisticsDO.setMinTime(statisticsDO1.getMinTime() < statisticsDO2
                    .getMinTime() ? statisticsDO1.getMinTime() : statisticsDO2.getMinTime());
            statisticsDO.setAvgTime((statisticsDO1.getAvgTime() + statisticsDO2.getAvgTime()) / 2);
            return statisticsDO;
        }

    }

    public StatisticsDO[] getSingleServerStatistics(String serverID) throws ServiceBusException {
        Object inPSStatisticsSummary = StatisticsDBUtils
                .getSummaryStatisticsForServer(pm, serverID, 0, Constants.PROXYSERVICE_STATISTICS);
        Object outStatisticsSummary = StatisticsDBUtils
                .getSummaryStatisticsForServer(pm, serverID, 1, Constants.PROXYSERVICE_STATISTICS);
        StatisticsDO[] statisticsDOs = null;
        StatisticsDO inStatisticsDO = null;
        StatisticsDO ouStatisticsDO = null;
        if (inPSStatisticsSummary instanceof Object[]) {
            inStatisticsDO = listToStatisticsDO((Object[]) inPSStatisticsSummary);
            inStatisticsDO.setServerId(serverID);
            inStatisticsDO.setDirection(0);

        }
        if (outStatisticsSummary instanceof Object[]) {
            ouStatisticsDO = listToStatisticsDO((Object[]) outStatisticsSummary);
            ouStatisticsDO.setServerId(serverID);
            ouStatisticsDO.setDirection(1);
        }
        if (inStatisticsDO != null && ouStatisticsDO != null) {
            statisticsDOs = new StatisticsDO[2];
            statisticsDOs[0] = inStatisticsDO;
            statisticsDOs[1] = ouStatisticsDO;
        } else if (inStatisticsDO == null && ouStatisticsDO != null) {
            statisticsDOs = new StatisticsDO[1];
            statisticsDOs[0] = ouStatisticsDO;
        } else if (inStatisticsDO != null) {
            statisticsDOs = new StatisticsDO[1];
            statisticsDOs[0] = inStatisticsDO;
        }
        return statisticsDOs;
    }

    public String echoServerId(String serverID) {
        return serverID;
    }

    public String[] listServers() throws ServiceBusException {
        Object namesObject = StatisticsDBUtils.getServersIds(pm);
        String[] names = new String[0];
        if (namesObject instanceof List) {
            List idObjects = (List) namesObject;
            names = new String[idObjects.size()];
            for (int i = 0; i < idObjects.size(); i++) {
                names[i] = (String) idObjects.get(i);
            }
        } else if (namesObject != null && namesObject instanceof String) {
            names = new String[]{(String) namesObject};
        }
        return names;
    }

    public String[] listSequence() throws ServiceBusException {
        Object namesObject = StatisticsDBUtils.getUniqueNames(pm, Constants.SEQUENCE_STATISTICS);
        String[] names = new String[0];
        if (namesObject instanceof List) {
            List idObjects = (List) namesObject;
            names = new String[idObjects.size()];
            for (int i = 0; i < idObjects.size(); i++) {
                names[i] = (String) idObjects.get(i);
            }
        } else if (namesObject != null && namesObject instanceof String) {
            names = new String[]{(String) namesObject};
        }
        return names;
    }

    public String[] listProxyServices() throws ServiceBusException {
        Object namesObject = StatisticsDBUtils.getUniqueNames(pm, Constants.PROXYSERVICE_STATISTICS);
        String[] names = new String[0];
        if (namesObject instanceof List) {
            List idObjects = (List) namesObject;
            if (idObjects.contains("SynapseService")) {
                names = new String[idObjects.size() - 1];
            } else {
                names = new String[idObjects.size()];
            }
            int index = 0;
            for (int i = 0; i < idObjects.size(); i++) {
                String name = (String) idObjects.get(i);
                if (!name.equals("SynapseService")) {
                    names[index] = name;
                    index++;
                }
            }
        } else
        if (namesObject != null && namesObject instanceof String && !(namesObject.equals("SynapseService")))
        {
            names = new String[]{(String) namesObject};
        }
        return names;

    }

    public GraphData getDataForGraph() throws ServiceBusException {
        GraphData graphData = new GraphData();
        List serverData = StatisticsDBUtils.getTotalCountForServerS(pm, Constants.PROXYSERVICE_STATISTICS);
        List psData = StatisticsDBUtils.getTotalCountForCategory(pm, Constants.PROXYSERVICE_STATISTICS);
        List epData = StatisticsDBUtils.getTotalCountForCategory(pm, Constants.ENDPOINT_STATISTICS);
        List sequenceData = StatisticsDBUtils.getTotalCountForCategory(pm, Constants.SEQUENCE_STATISTICS);
        String serverDataStr = "";
        String psDataStr = "";
        String epDataStr = "";
        String sequenceDataStr = "";

        for (Iterator iterator = serverData.iterator(); iterator.hasNext();) {
            Object[] row = (Object[]) iterator.next();
            serverDataStr += row[0] + "-[" + row[1] + "]-," + row[1] + ";";
        }
        for (Iterator iterator = psData.iterator(); iterator.hasNext();) {
            Object[] row = (Object[]) iterator.next();
            String name = (String) row[0];
            if (!name.equals("SynapseService")) {
                psDataStr += row[0] + "-[" + row[1] + "]-," + row[1] + ";";
            }
        }
        for (Iterator iterator = epData.iterator(); iterator.hasNext();) {
            Object[] row = (Object[]) iterator.next();
            epDataStr += row[0] + "-[" + row[1] + "]-," + row[1] + ";";
        }
        for (Iterator iterator = sequenceData.iterator(); iterator.hasNext();) {
            Object[] row = (Object[]) iterator.next();
            sequenceDataStr += row[0] + "-[" + row[1] + "]-," + row[1] + ";";
        }

        graphData.setEndPointData(epDataStr);
        graphData.setProxyServiceData(psDataStr);
        graphData.setSequenceData(sequenceDataStr);
        graphData.setServerData(serverDataStr);
        return graphData;
    }

    public String[] listEndPoint() throws ServiceBusException {
        Object namesObject = StatisticsDBUtils.getUniqueNames(pm, Constants.ENDPOINT_STATISTICS);
        String[] names = new String[0];
        if (namesObject instanceof List) {
            List idObjects = (List) namesObject;
            names = new String[idObjects.size()];
            for (int i = 0; i < idObjects.size(); i++) {
                names[i] = (String) idObjects.get(i);
            }
        } else if (namesObject != null && namesObject instanceof String) {
            names = new String[]{(String) namesObject};
        }
        return names;
    }

    public StatisticsData getServerProxyStatistics(String serverID) throws ServiceBusException {
        StatisticsData statisticsData = getData(Constants.PROXYSERVICE_STATISTICS, serverID);
        StatisticsDO[] statisticsDOs = statisticsData.getStatisticsRecords();
        boolean isContainSynapseService = false;
        for(int i=0;i<statisticsDOs.length;i++){
           StatisticsDO statisticsDO = statisticsDOs[i];
            if (statisticsDO != null && (statisticsDO.getName().equals("SynapseService"))) {
               isContainSynapseService = true;
               break;
            }
        }
        StatisticsDO[] resultSet = new StatisticsDO[statisticsDOs.length];
        if(isContainSynapseService) {
           resultSet = new StatisticsDO[statisticsDOs.length - 2];
        }
        int i = 0;
        int index = 0;
        while (statisticsDOs.length > i) {
            StatisticsDO statisticsDO = statisticsDOs[i];
            if (statisticsDO != null && !(statisticsDO.getName().equals("SynapseService"))) {
                resultSet[index] = statisticsDO;
                index++;
            }
            i++;
        }
        statisticsData.setStatisticsRecords(resultSet);
        return statisticsData;
    }

    public StatisticsDO[] getProxyServiceStatistics(String proxyName) throws ServiceBusException {
        return getRecords(Constants.PROXYSERVICE_STATISTICS, proxyName);
    }

    public StatisticsData getServerSequenceStatistics(String serverID) throws ServiceBusException {
        return getData(Constants.SEQUENCE_STATISTICS, serverID);
    }

    public StatisticsDO[] getSequenceStatistics(String sequenceName) throws ServiceBusException {
        return getRecords(Constants.SEQUENCE_STATISTICS, sequenceName);
    }

    public StatisticsData getServerEndPointStatistics(String serverID) throws ServiceBusException {
        return getData(Constants.ENDPOINT_STATISTICS, serverID);
    }

    public StatisticsDO[] getEndPointStatistics(String endPointName) throws ServiceBusException {
        return getRecords(Constants.ENDPOINT_STATISTICS, endPointName);
    }

    public StatisticsData getServerMessageMediationStatistics(String serverID) throws ServiceBusException {
        return getMessageMediationData(serverID);
    }

    public StatisticsDO[] getSingleStatistics(String name, int type) throws ServiceBusException {
        switch (type) {
            case Constants.SEQUENCE_STATISTICS : {
                return getSequenceStatistics(name);
            }
            case Constants.ENDPOINT_STATISTICS : {
                return getEndPointStatistics(name);
            }
            case Constants.PROXYSERVICE_STATISTICS: {
                return getProxyServiceStatistics(name);
            }
            default : {
                return new StatisticsDO[1];
            }
        }
    }

    public int  clearAllStatistics(){
        return StatisticsDBUtils.clearAll(pm);
    }
}
