package org.wso2.esb.transport.jetty;

import org.mortbay.jetty.handler.AbstractHandler;
import org.mortbay.jetty.Request;
import org.mortbay.jetty.HttpConnection;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.context.SessionContext;
import org.apache.axis2.transport.http.TransportHeaders;
import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.axis2.*;
import org.apache.axis2.Constants;
import org.apache.axis2.engine.AxisEngine;
import org.apache.axis2.util.UUIDGenerator;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.description.TransportOutDescription;
import org.apache.axis2.i18n.Messages;
import org.apache.axiom.soap.SOAPFaultCode;
import org.apache.axiom.soap.SOAP12Constants;
import org.apache.axiom.om.OMElement;
import org.wso2.utils.ServerConfiguration;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.ServletException;
import javax.xml.namespace.QName;
import java.util.Map;
import java.util.HashMap;
import java.util.Enumeration;
import java.io.IOException;
import java.io.OutputStream;

public abstract class Axis2Handler extends AbstractHandler {

    protected static final Log log = LogFactory.getLog(Axis2Handler.class);
    protected ConfigurationContext configurationContext;


    public Axis2Handler(ConfigurationContext configurationContext) {
        this.configurationContext = configurationContext;
    }

    protected Map getHeaders(HttpServletRequest request) {
        HashMap headerMap = new HashMap();
        Enumeration e = request.getAttributeNames();//TODO .getFieldNames();
        while (e.hasMoreElements()) {
            String field = (String) e.nextElement();
            headerMap.put(field, request.getAttribute(field)); // TODO .getField(field)
        }
        return headerMap;
    }

    protected Map getTransportHeaders(HttpServletRequest req) {
        return new TransportHeaders(req);
    }

    /**
     * New documentation
     *
     * @param target
     * @param request
     * @param response
     * @param dispatch
     * @throws java.io.IOException
     * @throws javax.servlet.ServletException
     */
    public void handle(String target, HttpServletRequest request, HttpServletResponse response,
                       int dispatch) throws IOException, ServletException {

        if (configurationContext == null) {
            throw new AxisFault(Messages
                    .getMessage("cannotBeNullConfigurationContext"));
        }
        Request baseRequest = (request instanceof Request) ? (Request) request :
                              HttpConnection.getCurrentConnection().getRequest();
        baseRequest.setHandled(true);

        OutputStream bos = response.getOutputStream();
        TransportOutDescription transportOut =
                configurationContext.getAxisConfiguration().
                        getTransportOut(org.apache.axis2.Constants.TRANSPORT_HTTP);
        MessageContext msgContext = new MessageContext();

        String trsPrefix = request.getRequestURL().toString();
        int sepindex = trsPrefix.indexOf(':');
        if (sepindex >= 0) {
            trsPrefix = trsPrefix.substring(0, sepindex);
            msgContext.setIncomingTransportName(trsPrefix);
        } else {
            msgContext.setIncomingTransportName(Constants.TRANSPORT_HTTP);
        }
        msgContext.setConfigurationContext(configurationContext);
        msgContext.
                setTransportIn(configurationContext.getAxisConfiguration().
                        getTransportIn(Constants.TRANSPORT_HTTP));
        msgContext.setTransportOut(transportOut);
        msgContext.setServerSide(true);

        String requestURI = request.getRequestURI();
        ServerConfiguration serverConfig = ServerConfiguration.getInstance();
        requestURI =
                requestURI.replaceFirst(serverConfig.getFirstProperty("Axis2Config.RESTContext"),
                                        serverConfig.getFirstProperty("Axis2Config.SOAPContext"));
        msgContext.setTo(new EndpointReference(requestURI));

        msgContext.setProperty(Constants.OUT_TRANSPORT_INFO,
                               new JettyHTTPOutTransportInfo(response));

        msgContext.setProperty(MessageContext.TRANSPORT_OUT, bos);

        // set the transport Headers
        msgContext.setProperty(MessageContext.TRANSPORT_HEADERS,
                               getTransportHeaders(request));
//        msgContext.setServiceGroupContextId(UUIDGenerator.getUUID());

        // set msg context properties
        msgContext.setProperty(MessageContext.REMOTE_ADDR, request.getRemoteAddr());
        // set request to context properties
        msgContext.setProperty(HTTPConstants.MC_HTTP_SERVLETREQUEST, request);

        msgContext.setFrom(new EndpointReference(request.getRemoteAddr()));

        /*Session Management for Admin Services*/
        manageSessionContext(request, response, msgContext);

        String method = request.getMethod();
        try {
            if ("GET".equals(method))
            { // HttpServletRequest.this. __GET
                boolean processed = processGetRequest(msgContext, request, response);
                if (!processed) {
                    response.setStatus(200);
                }
            } else
            if ("POST".equals(method)) {

                if (configurationContext == null) {
                    throw new AxisFault(Messages
                            .getMessage("cannotBeNullConfigurationContext"));
                }
                boolean processed = processPostRequest(msgContext, request, response);

                if (msgContext.getOperationContext() == null) {
                    response.setStatus(202);
                } else {
                    Object contextWritten =
                            msgContext.getOperationContext()
                                    .getProperty(Constants.RESPONSE_WRITTEN);

                    if ((contextWritten == null) || !Constants.VALUE_TRUE.equals(contextWritten)) {
                        response.setStatus(HttpServletResponse.SC_ACCEPTED);
                    } else if (processed) {
                        response.setStatus(200);
                    }
                }

            } else {
                log.debug("Unsupported: not POST or GET");
                response.sendError(
                        HttpServletResponse.SC_NOT_IMPLEMENTED);//HttpResponse.__501_Not_Implemented);
            } // not GET or POST
        } catch (AxisFault e) {
            //TODO null check for instance of MessageContext
            try {
                AxisEngine engine = new AxisEngine(configurationContext);
                msgContext.setProperty(MessageContext.TRANSPORT_OUT, bos);
                MessageContext faultContext =
                        engine.createFaultMessageContext(msgContext, e);

                response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);

                SOAPFaultCode code = faultContext.getEnvelope().getBody().getFault().getCode();
                OMElement valueElement = null;
                if (code != null) {
                    valueElement = code.getFirstChildWithName(new QName(
                            SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI,
                            SOAP12Constants.SOAP_FAULT_VALUE_LOCAL_NAME));
                }

                if (valueElement != null) {
                    if (valueElement.getText().trim().indexOf(
                            SOAP12Constants.FAULT_CODE_SENDER) > -1) {
                        response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
                    }
                }

                String state =
                        (String) msgContext.getProperty(
                                org.wso2.esb.transport.jetty.Constants.HTTP_RESPONSE_STATE);
                if (state != null) {
                    int stateInt = Integer.parseInt(state);
                    response.setStatus(stateInt);
                    if (stateInt == 401) { // Unauthorized
                        response.addHeader("WWW-Authenticate",
                                           "basic realm=\"WSO2 ESB Users\"");
                    }
                }

                engine.sendFault(faultContext);
            } catch (Throwable t) {
                log.error(t.getMessage(), t);
                response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            }
            String msg = e.getMessage();
            if (msg != null && msg.toLowerCase().indexOf("invalid service group id") == -1) {
                log.error("Unable to handle request", e);
            }
        }
    }

    /**
     * This method will determine, which requests should be injected with SessionContext.
     * @param request
     * @param response
     * @param msgContext
     * @throws ServletException
     */
    protected void manageSessionContext(HttpServletRequest request, HttpServletResponse response,
                                        MessageContext msgContext) throws ServletException {

        HttpSession httpSession = request.getSession(false);

        if (httpSession != null) {
            SessionContext sessionContext =
                    (SessionContext) httpSession.getAttribute(Constants.SESSION_CONTEXT_PROPERTY);
            if (sessionContext == null && httpSession.isNew()) {
                if (!explicitlyIngnorePaths(request)) {
                    String cookieValueString = httpSession.getId();
                    sessionContext = new SessionContext(null);
                    sessionContext.setCookieID(cookieValueString);
                    httpSession.setAttribute(Constants.SESSION_CONTEXT_PROPERTY, sessionContext);
                    msgContext.setSessionContext(sessionContext);
                }
            }
        }

    }

    /**
     * TODO This is a temp method; that will used to temporary supress creation of SessionContexts
     *
     * @param request
     * @return bolean
     * @depricate
     */
    private boolean explicitlyIngnorePaths(HttpServletRequest request) {
        String matchingString = "statistics";
        return request.getRequestURI().indexOf(matchingString) > -1;

    }


    protected abstract boolean processPostRequest(MessageContext msgContext,
                                                  HttpServletRequest request,
                                                  HttpServletResponse response)
            throws AxisFault;

    protected abstract boolean processGetRequest(MessageContext msgContext,
                                                 HttpServletRequest request,
                                                 HttpServletResponse response)
            throws AxisFault;

}
