/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.esb.transport.jetty;

import org.apache.axiom.om.OMElement;
import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.mortbay.jetty.Connector;
import org.mortbay.jetty.Server;
import org.mortbay.jetty.security.SslSocketConnector;
import org.wso2.esb.ServiceBusManager;
import org.wso2.utils.transport.AbstractTransportListener;
import org.wso2.utils.transport.http.HttpServerManager;

import javax.xml.namespace.QName;
/*
 * 
 */

public class HttpsTransportListener extends AbstractTransportListener {

    private static Log log = LogFactory.getLog(HttpsTransportListener.class);


    public EndpointReference getEPRForService(String serviceName,
                                              String ip) throws AxisFault {
        return getEPR("https", serviceName, ip);
    }

    public EndpointReference[] getEPRsForService(String serviceName, String ip) throws AxisFault {
        EndpointReference soapEPR = getEPR("https", serviceName, ip);
        return new EndpointReference[]{getEPRForService(serviceName, ip)};
    }

    public void destroy() {
        // TODO
    }

    public void start() throws AxisFault {

        Server httpsServer = HttpServerManager
            .getServer(Constants.HTTPS_SERVER);
        SslSocketConnector sslSocketConnector = new SslSocketConnector();
        sslSocketConnector.setPort(port);
        ServiceBusManager.httpsPort = port;
        int maxIdleTimeMs
                = Integer.parseInt(tInDescription.
                getParameter("max.idle.time.ms").getValue().toString());
        int lowResources
                = Integer.parseInt(tInDescription.
                getParameter("low.resources").getValue().toString());
        int lowResourcesPersistTimeMs
                = Integer.parseInt(tInDescription.
                getParameter("low.resource.persist.time.ms")
                .getValue().toString());
        OMElement ksEle =
                tInDescription.getParameter("keystore").getParameterElement()
                        .getFirstElement();
        String ksLocation =
                ksEle.getFirstChildWithName(new QName(Constants.ESB_NAMESPACE_URI,"Location")).getText();
        String ksType =
                ksEle.getFirstChildWithName(new QName(Constants.ESB_NAMESPACE_URI,"Type")).getText();
        String ksPassword =
                ksEle.getFirstChildWithName(new QName(Constants.ESB_NAMESPACE_URI,"Password")).getText();
        String pvtKeyPassword =
                ksEle.getFirstChildWithName(new QName(Constants.ESB_NAMESPACE_URI,"KeyPassword")).getText();

        sslSocketConnector.setMaxIdleTime(maxIdleTimeMs);

        sslSocketConnector.setKeystore(ksLocation);
        sslSocketConnector.setKeystoreType(ksType);
        sslSocketConnector.setPassword(ksPassword);
        sslSocketConnector.setKeyPassword(pvtKeyPassword);

        httpsServer.setConnectors(new Connector[]{sslSocketConnector});

        log.info("HTTPS listener starting on port " + port);
    }
}
