package org.wso2.esb.transport.jetty;

import org.mortbay.jetty.handler.AbstractHandler;
import org.mortbay.jetty.Request;
import org.mortbay.jetty.HttpConnection;
import org.mortbay.log.Log;
import org.wso2.utils.transport.http.FaviconProvider;
import org.wso2.utils.NetworkUtils;
import org.wso2.esb.util.JettyWebConfiguration;
import org.wso2.esb.ServiceBusManager;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import java.io.IOException;
import java.net.SocketException;

/**
 * If users tried to use of any root context other than the registred contextes in server-web.xml
 * or axis2.xml, this handler will invoke. Since this is Jetty related, all logger etc. are sync with
 * Jetty symantics.
 */
public class ServerDefaultHandler extends AbstractHandler {

    private static JettyWebConfiguration jettyConfig = JettyWebConfiguration.getInstance();


    private FaviconProvider faviconProvider;

    public ServerDefaultHandler() {
        try {
            faviconProvider = new FaviconProvider();
            faviconProvider.setFavIconFromResource("org/wso2/esb/transport/jetty/favicon.ico");
        } catch (Exception e) {
            Log.warn(e);
        }
    }


    public void handle(String target, HttpServletRequest request, HttpServletResponse response,
                       int dispatch) throws IOException, ServletException {
        Request baseRequest = request instanceof Request ? (Request) request :
                              HttpConnection.getCurrentConnection().getRequest();
        if (response.isCommitted() || baseRequest.isHandled()) {
            return;
        }
        baseRequest.setHandled(true);

        if (faviconProvider.provideFavIcon(request, response)) {
            return;
        }

        try {
            response.sendRedirect("https://" + NetworkUtils.getLocalHostname() +
                    ":" + ServiceBusManager.httpsPort + jettyConfig.adminWebCtx);
        } catch (SocketException e) {
            throw new ServletException(e);
        }
    }
}
