package org.wso2.esb.util;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.axiom.om.xpath.AXIOMXPath;
import org.apache.axiom.om.OMElement;
import org.wso2.utils.ServerException;
import org.wso2.esb.Constants;
import org.jaxen.SimpleNamespaceContext;
import org.jaxen.XPath;
import org.jaxen.JaxenException;

import javax.xml.stream.XMLStreamException;
import java.io.FileNotFoundException;
import java.util.List;

public class XmlConfiguration {

    private static Log log = LogFactory.getLog(XmlConfiguration.class);

    private StAXOMBuilder builder;

    private String serverNamespace = Constants.ESB_XML_NAMESPACE;

    public XmlConfiguration(String xmlFile) throws ServerException {
        this(xmlFile, null);
    }

    public XmlConfiguration(String xmlFile, String serverNamespace) throws ServerException {
        if (serverNamespace != null) {
            this.serverNamespace = serverNamespace;
        }
        try {
            builder = new StAXOMBuilder(xmlFile);
        } catch (XMLStreamException e) {
            log.error(e);
            throw new ServerException(e);
        } catch (FileNotFoundException e) {
            log.error(e);
            throw new ServerException(e);
        }
    }

    public String getUniqueValue(String xPath) {
        SimpleNamespaceContext nsCtx = new SimpleNamespaceContext();
        nsCtx.addNamespace("ns", serverNamespace);
        try {
            XPath xp = new AXIOMXPath(xPath);
            xp.setNamespaceContext(nsCtx);
            OMElement elem = builder.getDocumentElement();
            if (elem != null) {
                List nodeList = xp.selectNodes(elem);
                Object obj;
                if (!nodeList.isEmpty() && ((obj = nodeList.get(0)) != null)) {
                    return ((OMElement) obj).getText();
                }
            }
        } catch (JaxenException e) {
            throw new RuntimeException("XPath expression " + xPath + " failed", e);
        }
        return null;
    }

    public OMElement[] getElements(String xPath) {
        SimpleNamespaceContext nsCtx = new SimpleNamespaceContext();
        nsCtx.addNamespace("ns", serverNamespace);
        try {
            XPath xp = new AXIOMXPath(xPath);
            xp.setNamespaceContext(nsCtx);
            OMElement elem = builder.getDocumentElement();
            if (elem != null) {
                List nodeList = xp.selectNodes(elem);
                return (OMElement[]) nodeList.toArray(new OMElement[nodeList.size()]);
            }
        } catch (JaxenException e) {
            throw new RuntimeException("XPath expression " + xPath + " failed", e);
        }
        return new OMElement[0];
    }
}
