/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.esb.util;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.description.AxisServiceGroup;
import org.apache.axis2.rpc.receivers.RPCMessageReceiver;
import org.wso2.esb.ServiceBus;
import org.wso2.esb.ServiceBusException;
import org.wso2.esb.services.LoginAdmin;

import javax.xml.namespace.QName;

/*
*
*/

public class ServiceBusUtils {
    private ServiceBus esb;

    public static void deployService(
            ConfigurationContext configurationContext, AxisService service)
            throws AxisFault {
        configurationContext.getAxisConfiguration().addService(service);
    }

    public static void deployService(
            ConfigurationContext configurationContext, String serviceImplClass,
            Class messageReceiverClass) throws AxisFault {
        AxisService simpleService = AxisService.createService(serviceImplClass,
                configurationContext.getAxisConfiguration(),
                messageReceiverClass);
        configurationContext.getAxisConfiguration().addService(simpleService);
    }

    public static AxisServiceGroup deployService(
            ConfigurationContext configurationContext, AxisServiceGroup asg,
            String serviceImplClass, Class messageReceiverClass,
            String targetNameSpace, String schemaNamespace)
            throws AxisFault {
        AxisService simpleServiceWithSchemaSetting = AxisService.createService(serviceImplClass,
                configurationContext.getAxisConfiguration(),
                messageReceiverClass, targetNameSpace, schemaNamespace);
        simpleServiceWithSchemaSetting.setScope("soapsession");

        asg.addService(simpleServiceWithSchemaSetting);

        return asg;
    }

    public void startServer() throws AxisFault, ServiceBusException {
        try {
            //setting the System properties
            System.setProperty("axis2.xml", "conf/axis2.xml");
            System.setProperty("axis2.repo", "target/test-repo/repository");
            System.setProperty("synapse.xml", "test-resources/synapse.xml");

            //setting up the Testing port
            esb = new ServiceBus();

            esb.startListener(TestConstants.TESTING_PORT);
            Thread.sleep(1000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    public void stopServer() throws ServiceBusException {
        if (esb != null) {
            esb.stopServer();
        }
    }

    public synchronized void unDeployService(
            ConfigurationContext configurationContext, QName service)
            throws AxisFault {
        configurationContext.getAxisConfiguration().removeService(service.getLocalPart());
    }

    public ServiceBus getServiceBus() {
        return this.esb;
    }

    public static AxisServiceGroup deployLoginService(
            ConfigurationContext configurationContext, AxisServiceGroup asg)
            throws AxisFault {
        return ServiceBusUtils.deployService(configurationContext, asg,
                LoginAdmin.class.getName(), RPCMessageReceiver.class, "",
                "http://org.apache.synapse/xsd");
    }
}
